
/*
 * borg-classifier/conf.c: configuration file read-in source code
 * Copyright (C) 2011 Bradley Worley
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * 
 *     Free Software Foundation, Inc.
 *     59 Temple Place, Suite 330
 *     Boston, MA 02111-1307 USA
 */

/* include the core header. */
#include "core.h"

/* config key/value pairs. */
unsigned int n_conf;
char **conf_keys;
char **conf_vals;

/* conf_init: initializes the conf file.
 */
int conf_init (const char *fname) {
  /* declare some needed variables. */
  FILE *fh;
  unsigned int i;
  char buf[80], key[256], val[256];
  char *bytes;

  /* print a nice message. */
  fprintf (stderr, "INFO: reading in configuration variables.\n");

  /* initialize the key/value pairs. */
  n_conf = 0;
  conf_keys = NULL;
  conf_vals = NULL;

  /* open the config file. */
  fh = fopen (fname, "r");
  if (!fh)
    return 1;

  /* loop through the lines of the config file. */
  while (!feof (fh)) {
    /* get the next line and increment the line count. */
    bytes = fgets (buf, 80, fh);
    n_conf++;
  }

  /* click the number of configs back one. */
  n_conf--;

  /* allocate space for storing key/value pairs. */
  conf_keys = (char **) malloc (sizeof (char *) * n_conf);
  conf_vals = (char **) malloc (sizeof (char *) * n_conf);

  /* loop through the confs. */
  for (i = 0; i < n_conf; i++) {
    /* null the arrays. */
    conf_keys[i] = NULL;
    conf_vals[i] = NULL;
  }

  /* rewind to the start of the file. */
  fseek (fh, 0, SEEK_SET);

  /* loop through the file, parsing this time. */
  i = 0;
  while (!feof (fh)) {
    /* get the current line. */
    if (fscanf (fh, "%256[^=]=%256[^\n] ", key, val) == 2) {
      /* allocate space for the keys and values. */
      conf_keys[i] = (char *) malloc (sizeof (char) * (strlen (key) + 1));
      conf_vals[i] = (char *) malloc (sizeof (char) * (strlen (val) + 1));

      /* copy the strings. */
      strcpy (conf_keys[i], key);
      strcpy (conf_vals[i], val);
    }

    /* increment the counter. */
    i++;
  }

  /* close the config file. */
  fclose (fh);
  return 0;
}

/* conf_get: accesses a variable from the config file
 * @var: name of the variable to access.
 */
char *conf_get (const char *var) {
  /* declare some needed variables. */
  unsigned int i;

  /* make sure we got the query string. */
  if (!conf_keys || !conf_vals)
    return NULL;

  /* loop through the key/value pairs. */
  for (i = 0; i < n_conf; i++) {
    /* check if this is the right string. */
    if (strcmp (conf_keys[i], var) == 0)
      return conf_vals[i];
  }

  /* didn't find it. */
  return NULL;
}

/* conf_check: accesses a variable from the config file
 * @var: name of the variable to access.
 */
void conf_check (const char *var) {
  /* declare some needed variables. */
  unsigned int i;

  /* print a nice message. */
  fprintf (stderr, "INFO: checking config variable '%s'.\n", var);

  /* make sure we have key/value pairs. */
  if (n_conf > 0 && conf_keys && conf_vals) {
    /* loop through the key/value pairs. */
    for (i = 0; i < n_conf; i++) {
      /* make sure the key is defined. */
      if (!conf_keys[i]) continue;

      /* check if this is the right string. */
      if (strcmp (conf_keys[i], var) == 0) return;
    }
  }

  /* didn't find it. */
  fprintf (stderr, "FAIL! unable to find conf key '%s'!\n", var);
  exit (1);
}

/* conf_null: checks if a conf value is NULL. */
int conf_null (const char *var) {
  /* get the conf val. */
  char *val = conf_get (var);

  /* see if it's NULL. */
  return !(strcmp (val, "NULL"));
}
