
/*
 * borg-classifier/core.h: core application header
 * Copyright (C) 2011 Bradley Worley
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * 
 *     Free Software Foundation, Inc.
 *     59 Temple Place, Suite 330
 *     Boston, MA 02111-1307 USA
 */

/* include libc headers. */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>
#include <time.h>
#include <pthread.h>
#include <sys/types.h>
#include <ctype.h>

/* configuration header. */
#include "config.h"

/* include mysql headers. */
#if CONFIGURE_MYSQL == 1
#  include <my_global.h>
#  include <my_sys.h>
#  include <mysql.h>
#endif

/* include mpi headers. 
#include "mpi.h"*/

/* gnu science library includes. */
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_eigen.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_math.h>

/* object headers. */
#include "conf.h"
#include "rand.h"
#include "db.h"
#include "str.h"
#include "util.h"
#include "stat.h"
#include "em.h"
#include "xlink.h"

/* define the minimum change that iterated values must experience. */
#define EPSILON  1.0e-6

/* array of uniprot id strings. */
char **uids;
unsigned long N;

/* array of uniprot id strings in biuids. */
char **biuids, **biuids_db;
unsigned long M;

/* array of dimension strings. */
char **dims;
unsigned long D;

/* array of dimension strings in bidims. */
char **bidims;
unsigned long R;

/* space, rows for each protein and columns for each dimension. */
signed int **space;

/* bispace. */
signed int **bispace;

/* distances between each protein. */
signed int **distances;
signed int max_dist;
double ave_dist, dev_dist;

/* the sets of known function proteins. */
unsigned long *mmp, n_mmp;
unsigned long *top, n_top;
unsigned long *egf, n_egf;
