
/*
 * borg-classifier/db.h: database interaction header
 * Copyright (C) 2011 Bradley Worley
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * 
 *     Free Software Foundation, Inc.
 *     59 Temple Place, Suite 330
 *     Boston, MA 02111-1307 USA
 */

#if CONFIGURE_MYSQL == 1

/* define mysql constants. */
#define MYSQL_HOSTNAME "bionmr-18"
#define MYSQL_USERNAME "bworley"
#define MYSQL_PASSWORD "mysqlpass"
#define MYSQL_DATABASE "borg_pdac"

/* define the two databases to pull from. */
#define DB_BORG_NP    "borg_np"
#define DB_BORG_PDAC  "borg_pdac"

/* define prefixes for dimensions. */
#define PRE_GOTERM   "goterm"
#define PRE_KDISEASE "keggd"
#define PRE_KPATHWAY "keggp"
#define PRE_KEYWORD  "keyw"
#define PRE_SEQSIM   "seqsim"
#define PRE_PFAM     "pfamily"
#define PRE_NOG      "enog"
#define PRE_PDBHET   "pdbhet"
#define PRE_OMA      "oma"
#define PRE_BINDING  "binding"
#define PRE_REG      "reg"
#define PRE_CANCER   "cancer"

/* define field calculations for dimensions. */
#define F_GOTERM   "concat('" PRE_GOTERM ":',GO.type,replace(GO.GO,'GO',''))"
#define F_KDISEASE "concat('" PRE_KDISEASE ":',keggDiseases.disease)"
#define F_KPATHWAY "concat('" PRE_KPATHWAY ":',keggPathways.pathway)"
#define F_KEYWORD  "concat('" PRE_KEYWORD ":',replace(upper(keywords.keyword),' ','-'))"
#define F_SEQSIM   "concat('" PRE_SEQSIM ":',replace(upper(seqsims.seqsim),' ','-'))"
#define F_PFAM     "concat('" PRE_PFAM ":',pfam.family)"
#define F_NOG      "concat('" PRE_NOG ":',eggNOG.eggNOG)"
#define F_PDBHET   "concat('" PRE_PDBHET ":',pdbLigands.HET)"
#define F_OMA      "concat('" PRE_OMA ":',OMA.omaGroup)"
#define F_BINDING  "concat('" PRE_BINDING ":',binding.monomerID)"
#define F_REG_UP   PRE_REG ":UP"
#define F_REG_DOWN PRE_REG ":DOWN"
#define F_CANCER_T PRE_CANCER ":TRUE"
#define F_CANCER_F PRE_CANCER ":FALSE"

/* define names of dimension categories. */
#define NAME_UID      "UID"
#define NAME_KEGG     "kegg"
#define NAME_GOTERM   "goterm"    /**/
#define NAME_KDISEASE "keggd"     /**/
#define NAME_KPATHWAY "keggp"     /**/
#define NAME_KEYWORD  "keyword"   /**/
#define NAME_SEQSIM   "seqsim"    /**/
#define NAME_PFAM     "pfamily"   /**/
#define NAME_NOG      "nog"       /**/
#define NAME_PDBHET   "HET"       /**/
#define NAME_PDB      "PDB"
#define NAME_OMA      "oma"       /**/
#define NAME_BINDING  "bindingdb" /**/
#define NAME_REG      "regulation"
#define NAME_REG_UP   F_REG_UP
#define NAME_REG_DOWN F_REG_DOWN
#define NAME_CANCER   "cancer"
#define NAME_CANCER_T F_CANCER_T
#define NAME_CANCER_F F_CANCER_F

/* define tables from which to pull dimensions. */
#define TBL_UNIPROT  "uniprot"
#define TBL_GOTERM   "GO"
#define TBL_KEGG     "kegg"
#define TBL_KDISEASE "keggDiseases"
#define TBL_KPATHWAY "keggPathways"
#define TBL_KEYWORD  "keywords"
#define TBL_SEQSIM   "seqsims"
#define TBL_PFAM     "pfam"
#define TBL_NOG      "eggNOG"
#define TBL_OMA      "OMA"
#define TBL_BINDING  "binding"
#define TBL_PDB_PROT "pdbProteins"
#define TBL_PDB_LIG  "pdbLigands"
#define TBL_SOURCES  "sources"

/* define the goterm query for a protein. */
#define SQL_GOTERM \
 "select distinct " TBL_UNIPROT "." NAME_UID "," \
                    F_GOTERM " as " NAME_GOTERM \
 " from " TBL_UNIPROT "," TBL_GOTERM \
 " where " TBL_UNIPROT "." NAME_UID " = " TBL_GOTERM "." NAME_UID \
 "   and " TBL_UNIPROT "." NAME_UID " = '%s' %s;"

/* define the kegg disease query for a protein. */
#define SQL_KDISEASE \
 "select distinct " TBL_UNIPROT "." NAME_UID "," \
                    F_KDISEASE " as " NAME_KDISEASE \
 " from " TBL_UNIPROT "," TBL_KEGG "," TBL_KDISEASE \
 " where " TBL_UNIPROT "." NAME_UID " = " TBL_KEGG "." NAME_UID \
 "   and " TBL_KEGG "." NAME_KEGG " = " TBL_KDISEASE "." NAME_KEGG \
 "   and " TBL_UNIPROT "." NAME_UID " = '%s' %s;"

/* define the kegg pathway query for a protein. */
#define SQL_KPATHWAY \
 "select distinct " TBL_UNIPROT "." NAME_UID "," \
                    F_KPATHWAY " as " NAME_KPATHWAY \
 " from " TBL_UNIPROT "," TBL_KEGG "," TBL_KPATHWAY \
 " where " TBL_UNIPROT "." NAME_UID " = " TBL_KEGG "." NAME_UID \
 "   and " TBL_KEGG "." NAME_KEGG " = " TBL_KPATHWAY "." NAME_KEGG \
 "   and " TBL_UNIPROT "." NAME_UID " = '%s' %s;"

/* define the keyword query for a protein. */
#define SQL_KEYWORD \
 "select distinct " TBL_UNIPROT "." NAME_UID "," \
                    F_KEYWORD " as " NAME_KEYWORD \
 " from " TBL_UNIPROT "," TBL_KEYWORD \
 " where " TBL_UNIPROT "." NAME_UID " = " TBL_KEYWORD "." NAME_UID \
 "   and " TBL_UNIPROT "." NAME_UID " = '%s' %s;"

/* define the sequence similarity query for a protein. */
#define SQL_SEQSIM \
 "select distinct " TBL_UNIPROT "." NAME_UID "," \
                    F_SEQSIM " as " NAME_SEQSIM \
 " from " TBL_UNIPROT "," TBL_SEQSIM \
 " where " TBL_UNIPROT "." NAME_UID " = " TBL_SEQSIM "." NAME_UID \
 "   and " TBL_UNIPROT "." NAME_UID " = '%s' %s;"

/* define the pfam query for a protein. */
#define SQL_PFAM \
 "select distinct " TBL_UNIPROT "." NAME_UID "," \
                    F_PFAM " as " NAME_PFAM \
 " from " TBL_UNIPROT "," TBL_PFAM \
 " where " TBL_UNIPROT "." NAME_UID " = " TBL_PFAM "." NAME_UID \
 "   and " TBL_UNIPROT "." NAME_UID " = '%s' %s;"

/* define the eggnog query for a protein. */
#define SQL_NOG \
 "select distinct " TBL_UNIPROT "." NAME_UID "," \
                    F_NOG " as " NAME_NOG \
 " from " TBL_UNIPROT "," TBL_NOG \
 " where " TBL_UNIPROT "." NAME_UID " = " TBL_NOG "." NAME_UID \
 "   and " TBL_UNIPROT "." NAME_UID " = '%s' %s;"

/* define the pdbhet query for a protein. */
#define SQL_PDBHET \
 "select distinct " TBL_UNIPROT "." NAME_UID "," \
                    F_PDBHET " as " NAME_PDBHET \
 " from " TBL_UNIPROT "," TBL_PDB_PROT "," TBL_PDB_LIG \
 " where " TBL_UNIPROT "." NAME_UID " = " TBL_PDB_PROT "." NAME_UID \
 "   and " TBL_PDB_LIG "." NAME_PDB " = " TBL_PDB_PROT "." NAME_PDB \
 "   and " TBL_UNIPROT "." NAME_UID " = '%s' %s;"

/* define the oma query for a protein. */
#define SQL_OMA \
 "select distinct " TBL_UNIPROT "." NAME_UID "," \
                    F_OMA " as " NAME_OMA \
 " from " TBL_UNIPROT "," TBL_OMA \
 " where " TBL_UNIPROT "." NAME_UID " = " TBL_OMA "." NAME_UID \
 "   and " TBL_UNIPROT "." NAME_UID " = '%s' %s;"

/* define the binding db query for a protein. */
#define SQL_BINDING \
 "select distinct " TBL_BINDING "." NAME_UID "," \
                    F_BINDING " as " NAME_BINDING \
 " from " TBL_UNIPROT "," TBL_BINDING \
 " where " TBL_UNIPROT "." NAME_UID " = " TBL_BINDING "." NAME_UID \
 "   and " TBL_UNIPROT "." NAME_UID " = '%s' %s;"

/* define the up-regulation query for a protein. */
#define SQL_REG_UP \
 "select (" TBL_SOURCES "." NAME_REG " is not null" \
 " and " TBL_SOURCES "." NAME_REG " = 'up') as reg_UP" \
 " from " TBL_SOURCES "," TBL_UNIPROT \
 " where " TBL_UNIPROT "." NAME_UID " = " TBL_SOURCES "." NAME_UID \
 "   and " TBL_UNIPROT "." NAME_UID " = '%s' %s;"

/* define the up-regulation query for a protein. */
#define SQL_REG_DOWN \
 "select (" TBL_SOURCES "." NAME_REG " is not null" \
 " and " TBL_SOURCES "." NAME_REG " = 'down') as reg_DOWN" \
 " from " TBL_SOURCES "," TBL_UNIPROT \
 " where " TBL_UNIPROT "." NAME_UID " = " TBL_SOURCES "." NAME_UID \
 "   and " TBL_UNIPROT "." NAME_UID " = '%s' %s;"

/* query for matrix metalloproteins. */
#define Q_MMP \
 "select " TBL_UNIPROT "." NAME_UID " from " TBL_UNIPROT " where " \
 TBL_UNIPROT ".entry like '%%MMP%%' or " TBL_UNIPROT ".name like '%%metallo%%';"

/* query for topoisomerases. */
#define Q_TOP \
 "select UID from uniprot where " \
 "entry like '%%TOP%%' or name like '%%topoisomerase%%';"

/* query for endothelial growth factors. */
#define Q_EGF \
 "select UID from uniprot where " \
 "entry like '%%EGF%%' or name like '%%epidermal growth factor%%';"

/* db_results_t: holds mysql query results. */
struct db_results {
  unsigned long rows, cols;
  char **keys;
  char ***values;
};

/* declare functions. */
void db_connect (const char *database);
void db_disconnect (void);
int db_query (const char *format, ...);
struct db_results *db_results_get (void);
void db_results_free (struct db_results *results);
char *db_results_field (struct db_results *results,
                        unsigned int row,
                        const char *field);

#endif
