
/*
 * borg-classifier/em.h: expectation-maximization clustering header
 * Copyright (C) 2011 Bradley Worley
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * 
 *     Free Software Foundation, Inc.
 *     59 Temple Place, Suite 330
 *     Boston, MA 02111-1307 USA
 */

/* declare an expectation-maximization instance structure. */
struct em_inst {
  /* iteration counters. */
  unsigned long n_iter, n_init;

  /* cluster count. */
  unsigned long K;

  /* parameters of the clustering run. */
  double **mu;
  double ***sig;

  /* computed quantities of the clustering algorithm. */
  double eps;       /* change in log-likelihood between iterations. */
  double logL;      /* log-likelihood of the dataset for the model. */
  double *logdets;  /* log-determinants of of the covariance matrices. */
  double *Pk;       /* probability of randomly selecting a point from k. */
  double **p;       /* probabilities of selecting point i from cluster k. */

  /* intermediate quantities for the clustering algorithm. */
  double *dv, *dvv;

  /* computed qualities of the clustering result. */
  double W;
};

/* declare an expectation-maximization data matrix structure. */
struct em_data {
  /* data matrix and size values. */
  unsigned long m, n;
  double **d;
};

/* declare functions. */
struct em_data *em_data_new (void);
void em_data_free (struct em_data *X);
struct em_inst *em_inst_new (struct em_data *X);
void em_inst_free (struct em_inst *em, struct em_data *X);
int em_execute (struct em_inst *em, struct em_data *X);
void em_dispersion (struct em_inst *em, struct em_data *X);
void em_print_clustering (struct em_inst *em, struct em_data *X);
void em_print_parameters (struct em_inst *em, struct em_data *X);
