#!/bin/bash

taskdir=${1}
export GDFONTPATH=/usr/share/fonts/bitstream-vera

rm -f *.png

myk=10

for k in $(seq -w 1 ${myk}); do
  echo " processing k${k} labelled princomp"
  awk -f label.awk -F , \
    -v infile=../../${taskdir}/princomp.csv \
    ../../${taskdir}/best-cluster-k${k}.csv

  cat > l.plt << EOF
set terminal png enhanced font "VeraSeBd,80" size 3960,2970 linewidth 6
set encoding iso_8859_1

set xlabel "PC 1"
set ylabel "PC 2"
set zlabel "PC 3"

unset key

set output "l-2d-k${k}.png"
plot \\
EOF
  for kk in $(seq -w 1 ${myk}); do
    nk=$(expr ${kk} + 0)
    echo -n "'k${kk}.txt' with points ps 2 pt $(expr ${nk} % 8 + 1)" >> l.plt
    if [ "${kk}" == "${k}" ]; then
      echo >> l.plt
      break
    else
      echo ", \\" >> l.plt
    fi
  done

  cat >> l.plt << EOF
set output "c-2d-k${k}.png"
plot \\
EOF
  for kk in $(seq -w 1 ${myk}); do
    nk=$(expr ${kk} + 0)
    echo -n "'k${kk}.txt' with points ps 2 pt $(expr ${nk} % 8 + 1)" >> l.plt
    echo ", \\" >> l.plt
    if [ "${kk}" == "${k}" ]; then break; fi
  done
  echo "'../../${taskdir}/ellipse-k${k}.dat' w l lw 2 lt rgb \"#000000\"" >> l.plt

  cat >> l.plt << EOF
set output "l-3d-k${k}.png"
splot \\
EOF
  for kk in $(seq -w 1 ${myk}); do
    nk=$(expr ${kk} + 0)
    echo -n "'k${kk}.txt' with points ps 2 pt $(expr ${nk} % 8 + 1)" >> l.plt
    if [ "${kk}" == "${k}" ]; then
      echo >> l.plt
      break
    else
      echo ", \\" >> l.plt
    fi
  done

  gnuplot l.plt
  rm -f l.plt k*.txt
done

echo " running unlabelled 2d plot file"
sed -e "s,%TASKDIR%,${taskdir},g" u-2d.plt.in > u-2d.plt
gnuplot u-2d.plt
rm -f u-2d.plt

echo " running unlabelled 3d plot file"
sed -e "s,%TASKDIR%,${taskdir},g" u-3d.plt.in > u-3d.plt
gnuplot u-3d.plt
rm -f u-3d.plt

for pngfile in *.png; do
  echo " postprocessing ${pngfile}"
  convert -resize '50%' ${pngfile} tmp.png
  mv tmp.png ../../${taskdir}/princomp-${pngfile}
  rm -f ${pngfile}
done

