
/*
 * borg-classifier/gen-cluster.c: single clustering calculation app code
 * Copyright (C) 2011 Bradley Worley
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * 
 *     Free Software Foundation, Inc.
 *     59 Temple Place, Suite 330
 *     Boston, MA 02111-1307 USA
 */

/* core include. */
#include "core.h"

/* main: application entry point. */
int main (int argc, char **argv) {
  /* initialize the core structures. */
  util_init_core (argc, argv);

  /* allocate the data matrix and instance structures. */
  struct em_data *X = em_data_new ();
  struct em_inst *em = em_inst_new (X);

  /* run the clustering algorithm. */
  fprintf (stderr, "INFO: assigning %lu points to %lu clusters.\n",
           X->m, em->K);
  while (em_execute (em, X) < 0) {
    /* rebuild the clustering instance. */
    rand_init ();
    em_inst_free (em, X);
    em = em_inst_new (X);
  }

  /* print the clustering. */
  em_print_clustering (em, X);
  em_print_parameters (em, X);

  /* free the clustering instance and data matrix. */
  em_inst_free (em, X);
  em_data_free (X);

  /* exit the program. */
  return 0;
}
