
/*
 * borg-classifier/gen-distance.c: feature space cluster distancing app code
 * Copyright (C) 2011 Bradley Worley
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * 
 *     Free Software Foundation, Inc.
 *     59 Temple Place, Suite 330
 *     Boston, MA 02111-1307 USA
 */

/* core include. */
#include "core.h"

/* the number of clusters. */
unsigned long K;

/* the cluster index array. */
unsigned long *indices;
unsigned long **idx, *n_idx;

/* main: application entry point. */
int main (int argc, char **argv) {
  /* declare required variables. */
  unsigned long i, j, k, dist;
  double Dist;

  /* initialize the core structures. */
  util_init_core (argc, argv);

  /* check the conf keys. */
  conf_check (CONF_CLUSTER);
  conf_check (CONF_DISTANCE);

  /* read in the clustering file. */
  fprintf (stderr, "INFO: reading clustering from '%s'.\n",
           conf_get (CONF_CLUSTER));
  indices = read_clustering (conf_get (CONF_CLUSTER));

  /* loop through the index array to find the number of clusters. */
  for (K = 0, i = 0; i < N; i++) if (indices[i] > K) K = indices[i];
  K++;

  /* allocate some memory. */
  idx = (unsigned long **) malloc (sizeof (unsigned long *) * K);
  n_idx = (unsigned long *) malloc (sizeof (unsigned long) * K);
  for (k = 0; k < K; k++) {
    /* allocate the cluster indices. */
    idx[k] = (unsigned long *) malloc (sizeof (unsigned long) * N);
    n_idx[k] = 0;
  }

  /* loop through the points. */
  for (i = 0; i < N; i++) {
    /* store the cluster in its index. */
    k = indices[i];
    idx[k][n_idx[k]] = i;
    n_idx[k]++;
  }

  /* open the output file. */
  FILE *fout = fopen (conf_get (CONF_DISTANCE), "wb");

  /* loop through the clusters. */
  for (Dist = 0.0, k = 0; k < K; k++) {
    /* loop through the proteins in the cluster. */
    for (dist = 0, i = 0; i < n_idx[k]; i++) {
      /* add this protein's distance from the origin. */
      for (j = 0; j < D; j++) dist += space[idx[k][i]][j];
    }

    /* calculate the average distance. */
    Dist = (double) dist / (double) n_idx[k];

    /* output the distance. */
    fprintf (stderr, "INFO: (cluster %lu/%lu) average distance = %lf\n",
             k + 1, K, Dist);

    /* output the result. */
    fprintf (fout, "%lu,%lf\n", k, Dist);
  }

  /* close the output file. */
  fclose (fout);

  /* free the clustering indices. */
  free (indices);
  for (k = 0; k < K; k++) free (idx[k]);
  free (n_idx);
  free (idx);

  /* exit the program. */
  return 0;
}
