
/*
 * borg-classifier/gen-network.c: protein network creation app code
 * Copyright (C) 2011 Bradley Worley
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * 
 *     Free Software Foundation, Inc.
 *     59 Temple Place, Suite 330
 *     Boston, MA 02111-1307 USA
 */

/* core include. */
#include "core.h"

/* the number of clusters. */
unsigned long K;

/* the cluster index array. */
unsigned long *indices;
unsigned long **idx, *n_idx;

/* main: application entry point. */
int main (int argc, char **argv) {
#if CONFIGURE_MYSQL == 1
  /* declare required variables. */
  unsigned long i, j, k, n_edge, n_node;

  /* initialize the core structures. */
  util_init_core (argc, argv);

  /* check the conf keys. */
  conf_check (CONF_K);
  conf_check (CONF_CLUSTER);
  conf_check (CONF_NETWORK);

  /* read in the clustering file. */
  fprintf (stderr, "INFO: reading clustering from '%s'.\n",
           conf_get (CONF_CLUSTER));
  indices = read_clustering (conf_get (CONF_CLUSTER));

  /* loop through the index array to find the number of clusters. */
  for (K = 0, i = 0; i < N; i++) if (indices[i] > K) K = indices[i];
  K++;

  /* allocate some memory. */
  idx = (unsigned long **) malloc (sizeof (unsigned long *) * K);
  n_idx = (unsigned long *) malloc (sizeof (unsigned long) * K);
  for (k = 0; k < K; k++) {
    /* allocate the cluster indices. */
    idx[k] = (unsigned long *) malloc (sizeof (unsigned long) * N);
    n_idx[k] = 0;
  }

  /* connect to the database. */
  db_connect (NULL);
  n_edge = 0;
  n_node = 0;

  /* loop through the points. */
  for (i = 0; i < N; i++) {
    /* only include non-pdb structured proteins. */
    if (!uid_has_pdb (i)) {
      /* store the cluster in its index. */
      k = indices[i];
      idx[k][n_idx[k]] = i;
      n_idx[k]++;
    }
  }

  /* disconnect from the database. */
  db_disconnect ();

  /* get the number of clusters (k). */
  k = atol (conf_get (CONF_K)) - 1;

  /* open the output file. */
  FILE *fout = fopen (conf_get (CONF_NETWORK), "wb");

  /* loop through the proteins in the cluster. */
  int wrote;
  for (i = 0; i < n_idx[k]; i++) {
    /* assume we won't write any edges. */
    wrote = 0;

    /* and again, without duplication. */
    for (j = i + 1; j < n_idx[k]; j++) {
      /* only for 10 annotation differences or fewer. */
      if (distances[idx[k][i]][idx[k][j]] <= 10) {
        /* print the network line. */
        fprintf (fout, "%s,%s,%d\n", uids[i], uids[j],
                 distances[idx[k][i]][idx[k][j]]);

        /* increment the edge count. */
        n_edge++;
        wrote = 1;
      }
    }

    /* did we write edges? */
    if (wrote) n_node++;
  }

  /* close the output file. */
  fclose (fout);

  /* print a nice message. */
  fprintf (stdout,
           "INFO: (cluster %lu/%lu) wrote %lu edges between %lu nodes.\n",
           k + 1, K, n_edge, n_node);

  /* free the clustering indices. */
  free (indices);
  for (k = 0; k < K; k++) free (idx[k]);
  free (n_idx);
  free (idx);

  /* exit the program. */
  return 0;
#else
  /* fail out. */
  fprintf (stderr, "FAIL! %s can't run without a mysql connection!\n", argv[0]);
  return 1;
#endif
}
