
/*
 * borg-classifier/gen-pruned.c: pruned feature space calculation app code
 * Copyright (C) 2011 Bradley Worley
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * 
 *     Free Software Foundation, Inc.
 *     59 Temple Place, Suite 330
 *     Boston, MA 02111-1307 USA
 */

/* core include. */
#include "core.h"

/* main: application entry point. */
int main (int argc, char **argv) {
  /* declare required variables. */
  unsigned long Dhat, i, j, k;
  char *ofilename;
  FILE *ofile;

  /* initialize the core structures. */
  util_init_core (argc, argv);

  /* count the non-singleton dimensions. */
  fprintf (stderr, "INFO: counting useful dimensions... ");
  ofilename = (char *) malloc (sizeof (char) *
    (strlen (conf_get (CONF_DIMS)) + 8));
  sprintf (ofilename, "%s.p", conf_get (CONF_DIMS));
  ofile = fopen (ofilename, "wb");
  for (Dhat = 0, j = 0; j < D; j++) {
    /* do we have more than one member? */
    if (dim_count (j) > 1) {
      /* write to the output file. */
      fprintf (ofile, "%lu,%s\n", Dhat, dims[j]);

      /* increment the counter. */
      Dhat++;
    }
  }

  /* free the output filename. */
  fprintf (stderr, "%lu.\n", Dhat);
  free (ofilename);
  fclose (ofile);

  /* re-write the space file. */
  fprintf (stderr, "INFO: rewriting pruned space file.\n");
  ofilename = (char *) malloc (sizeof (char) *
    (strlen (conf_get (CONF_SPACE)) + 8));
  sprintf (ofilename, "%s.p", conf_get (CONF_SPACE));
  ofile = fopen (ofilename, "wb");
  for (i = 0; i < N; i++) {
    /* print the uid. */
    fprintf (ofile, "%lu,", i);

    /* and through the dims. */
    for (k = 0, j = 0; j < D; j++) {
      /* but only print useful dims. */
      if (dim_count (j) > 1) {
        /* write the dimension value. */
        fprintf (ofile, "%d", space[i][j]);
        k++;
      }
    }

    /* print the newline. */
    fprintf (ofile, "\n");
  }

  /* free the output filename. */
  free (ofilename);
  fclose (ofile);

  /* exit the program. */
  return 0;
}
