
/*
 * borg-classifier/gen-xlink-dip.c: DIP network creation app code
 * Copyright (C) 2011 Bradley Worley
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * 
 *     Free Software Foundation, Inc.
 *     59 Temple Place, Suite 330
 *     Boston, MA 02111-1307 USA
 */

/* ACHTUNG!!! this code is totally and pathetically incomplete. <3 brad. */

/* core include. */
#include "core.h"

/* the number of clusters. */
unsigned long K;

/* the cluster index array. */
unsigned long *indices;
unsigned long **idx, *n_idx;

/* main: application entry point. */
int main (int argc, char **argv) {
  /* declare required variables. */
  unsigned long i, j, k, len;

  /* initialize the core structures. */
  util_init_core (argc, argv);

  /* check the conf keys. */
  conf_check (CONF_K);
  conf_check (CONF_CLUSTER);
  conf_check (CONF_CONSENSUS);

  /* read in the clustering file. */
  fprintf (stderr, "INFO: reading clustering from '%s'.\n",
           conf_get (CONF_CLUSTER));
  indices = read_clustering (conf_get (CONF_CLUSTER));

  /* allocate the counts array. */
  totals = (unsigned long *) malloc (sizeof (unsigned long) * D);
  counts = (unsigned long *) malloc (sizeof (unsigned long) * D);
  sorted = (unsigned long *) malloc (sizeof (unsigned long) * D);
  percents = (double *) malloc (sizeof (double) * D);
  n_sorted = D;

  /* loop through the index array to find the number of clusters. */
  for (K = 0, i = 0; i < N; i++) if (indices[i] > K) K = indices[i];
  K++;

  /* allocate some memory. */
  idx = (unsigned long **) malloc (sizeof (unsigned long *) * K);
  n_idx = (unsigned long *) malloc (sizeof (unsigned long) * K);
  for (k = 0; k < K; k++) {
    /* allocate the cluster indices. */
    idx[k] = (unsigned long *) malloc (sizeof (unsigned long) * N);
    n_idx[k] = 0;
  }

  /* loop through the points. */
  for (i = 0; i < N; i++) {
    /* store the cluster in its index. */
    k = indices[i];
    idx[k][n_idx[k]] = i;
    n_idx[k]++;
  }

  /* loop through the dimensions. */
  for (j = 0; j < D; j++) {
    /* and through the clusters. */
    for (totals[j] = 0, k = 0; k < K; k++) {
      /* and through the proteins in our cluster. */
      for (i = 0; i < n_idx[k]; i++) {
        /* increment the dimension if we have an annotation in this protein. */
        if (space[idx[k][i]][j]) totals[j]++;
      }
    }
  }

  /* get the number of clusters (k). */
  k = atol (conf_get (CONF_K)) - 1;

  /* loop through the dimenions. */
  for (j = 0; j < D; j++) {
    /* and through the proteins in our cluster. */
    for (counts[j] = 0, i = 0; i < n_idx[k]; i++) {
      /* increment the dimension if we have an annotation in this protein. */
      if (space[idx[k][i]][j]) counts[j]++;
    }

    /* divide out the count to make a percent. */
    percents[j] = (double) counts[j] * 100.0 / (double) totals[j];
  }

  /* sort the counts array and count the number of enriched dimensions. */
  heapsort (k);
  for (len = 0, j = 0; j < D; j++)
    if (percents[j] > (100.0 / (double) K)) len++;

  /* open the output file. */
  FILE *fout = fopen (conf_get (CONF_CONSENSUS), "wb");
  fprintf (fout, CONSENSUS_HEADER, K, k + 1);

  /* loop through the sorted dimensions. */
  j = 0;
  db_connect (NULL);
  char *description;
  unsigned long s;
  while (n_sorted) {
    /* get the next one. */
    s = heapsort_get (k);

    /* for enriched dimensions only... */
    if (percents[s] > (100.0 / (double) K)) {
      /* get the description and print the line. */
      description = dim_desc (s);
      fprintf (fout, CONSENSUS_ENTRY, j % 2 ? "trGrey" : "trWhite",
               percents[s], counts[s], totals[s], s, dims[s], description);

      /* free the description string. */
      free (description);
    }

    /* increment the counter. */
    j++;

    /* check the counter for periodic status updates. */
    if (!(j % 250) && j <= len)
      fprintf (stderr,
        "INFO: (cluster %lu/%lu) writing consensus data (%.0lf%% done)\n",
        k + 1, K, (double) j / (double) len * 100.0);
  }

  /* close the output file. */
  db_disconnect ();
  fprintf (fout, CONSENSUS_FOOTER);
  fclose (fout);

  /* free the counts array. */
  free (totals);
  free (counts);
  free (sorted);
  free (percents);

  /* free the clustering indices. */
  free (indices);
  for (k = 0; k < K; k++) free (idx[k]);
  free (n_idx);
  free (idx);

  /* exit the program. */
  return 0;
}
