
/*
 * borg-classifier/str.c: string utility functions source code
 * Copyright (C) 2011 Bradley Worley
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * 
 *     Free Software Foundation, Inc.
 *     59 Temple Place, Suite 330
 *     Boston, MA 02111-1307 USA
 */

/* core include. */
#include "core.h"

/* str_dup: duplicates a string.
 * @stri: the input string.
 */
char *str_dup (const char *stri) {
  /* declare the output string. */
  char *stro;

  /* allocate memory and copy the input string. */
  stro = (char *) malloc (sizeof (char) * (strlen (stri) + 2));
  strcpy (stro, stri);

  /* return the new string. */
  return stro;
}

/* str_file: reads in the contents of a file into a string.
 * @filename: the filename of the file to read.
 */
char *str_file (const char *filename) {
  /* declare some needed variables. */
  FILE *fh;
  char *str;
  long fsize, bytes;

  /* try to open the file in question. */
  fh = fopen (filename, "r");
  if (!fh)
    return NULL;

  /* determine the number of chars in the file. */
  fseek (fh, 0, SEEK_END);
  fsize = ftell (fh);
  rewind (fh);

  /* allocate the string. */
  str = (char *) malloc (sizeof (char) * (fsize + 1));
  if (!str) {
    /* we failed to allocate memory. */
    fclose (fh);
    return NULL;
  }

  /* read in the file. */
  bytes = fread (str, 1, fsize, fh);
  if (bytes != fsize) {
    /* we failed to read in the whole file. */
    free (str);
    fclose (fh);
    return NULL;
  }

  /* make sure to null-terminate the string. */
  str[fsize] = '\0';

  /* return the string. */
  fclose (fh);
  return str;
}

/* str_freev: frees a string array. */
void str_freev (char **strv, unsigned long n) {
  /* declare required variables. */
  unsigned long i;

  /* loop through the length of the string array. */
  for (i = 0; i < n; i++)
    free (strv[i]);

  /* free the overall array. */
  free (strv);
}
