#!/bin/bash

# source the include file.
source task-include.sh

# get the task info.
taskname="fastcluster"
taskdate="${1}"
K=$(seq -f '%02g' ${2} ${2})

# get the task pwd and app.
gsl=/work/powers/bworley/gsl-1.14/dist
taskpwd=/work/powers/bworley/classifiers
taskapp=${taskpwd}/$(basename ${0})

# check the outputs directory.
if [ ! -d ${taskpwd}/outputs ]; then
  mkdir ${taskpwd}/outputs
  mkdir ${taskpwd}/outputs/conf
  mkdir ${taskpwd}/outputs/job
fi

# check the task date.
if [ "${taskdate}" == "" ]; then
  # we need a task directory.
  echo "FAIL! you must specify a task date as a first argument!" >&2
  exit 1
elif [ ! -d ${taskpwd}/outputs/${taskdate} ]; then
  # ask if we wish to create a new task directory.
  echo "FAIL! ${taskpwd}/outputs/${taskdate} does not exist!" >&2
  exit 1
fi

# set the task dir.
taskdir=${taskpwd}/outputs/${taskdate}
runconf=${taskpwd}/outputs/conf/${taskname}-${runtime}-${K}
jobconf=${taskpwd}/outputs/job/${taskname}-${runtime}-${K}
jobfile=${taskpwd}/outputs/job/${taskname}-${K}.lst
statfile=${taskpwd}/outputs/job/${taskname}-${K}.stat

function startcluster () {
  # get the run.
  run=${1}

  # make sure it doesnt exist.
  if [ -e ${taskdir}/cluster-k${K}-${run}.csv ]; then
    return
  fi

  # set up the conf file
  cat > ${runconf}-${run} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=NULL
dist=NULL
princomp=${taskdir}/princomp.csv
cluster=${taskdir}/cluster-k${K}-${run}.csv
parms=${taskdir}/parms-k${K}-${run}.csv
K=${K}
EOF

  # set up the job file.
  cat > ${jobconf}-${run} << EOF
#!/bin/sh
#PBS -N ${taskname}-${K}-${run}
#PBS -l select=1
#PBS -l walltime=06:00:00
#PBS -o ${jobconf}-${run}.out.txt
#PBS -e ${jobconf}-${run}.err.txt
export LD_LIBRARY_PATH=\${LD_LIBRARY_PATH}:${gsl}/usr/lib
${taskpwd}/gen-cluster.x ${runconf}-${run}
EOF

  # queue up the job.
  sync
  job=$(qsub ${jobconf}-${run})
  echo "${run}:${job}" >> ${jobfile}
}

function startclusters () {
  # loop through the run values, kicking off the 
  for run in $(seq -f '%03g' 1 ${myrun}); do
    # start the job if need-be.
    startcluster ${run}
  done
}

function jobid () {
  run=${1}
  grep ^${run} ${jobfile} | tail -n 1 | cut -f 2 -d ':'
}

function jobstatus () {
  jid=${1}
  status=$(grep ^${jid} ${statfile} | cut -f 2 -d ':')
  if [ "${status}" == "" ]; then status="C"; fi
  echo ${status}
}

function rundone () {
  run=${1}
  jid=$(jobid ${run})
  jst=$(jobstatus ${jid})
  if [ "${jst}" == "C" ]; then
    echo "y"
  else
    echo "n"
  fi
}

function jobstatuses () {
  # get the status of all jobs.
  qstat -u bworley | awk '/^[0-9]/{print$1":"$(NF-1)}' > ${statfile}
}

function alldone () {
  c=$(ls ${taskdir}/cluster-k${K}-*.csv 2>/dev/null | wc -l)
  if [ "${c}" == "${myrun}" ]; then
    echo "y"
  else
    echo "n"
  fi
}

echo "init"
startclusters
done=n

while [ ! "${done}" == "y" ]; do
  echo -n "zzz "
  sleep 2m

  echo -n "stat "
  jobstatuses

  echo -n "check "
  for run in $(seq -f '%03g' 1 ${myrun}); do
    if [ "$(rundone ${run})" == "y" ]; then
      if [ ! -e ${taskdir}/cluster-k${K}-${run}.csv ]; then
        echo -n "run "
        startcluster ${run}
      fi
    fi
  done

  echo -n "finished? "
  done=$(alldone)
  echo ${done}
done
