#!/bin/bash

# source the include file.
source task-include.sh

# get the task name.
taskname="${1}"
taskdate="${2}"

# see if we ran without arguments.
if [ "${taskname}" == "" -a "${taskdate}" == "" ]; then
  task_info_list
fi

# get the task pwd and app.
gsl=/work/powers/bworley/gsl-1.14/dist
taskpwd=/work/powers/bworley/classifiers
taskapp=${taskpwd}/$(basename ${0})

# check the outputs directory.
if [ ! -d ${taskpwd}/outputs ]; then
  mkdir ${taskpwd}/outputs
  mkdir ${taskpwd}/outputs/conf
  mkdir ${taskpwd}/outputs/job
fi

# check the task date.
if [ "${taskdate}" == "" ]; then
  # we need a task directory.
  echo "INFO: you must specify a task date as a second argument." >&2
  task_info ${taskname}
elif [ ! -d ${taskpwd}/outputs/${taskdate} ]; then
  # ask if we wish to create a new task directory.
  echo -n "${taskpwd}/outputs/${taskdate} does not exist. create? [n] "
  read resp

  # check the response.
  if [ "${resp}" == "y" -o "${resp}" == "Y" -o "${resp}" == "yes" ]; then
    # make the directory.
    mkdir ${taskpwd}/outputs/${taskdate}
  else
    # leave the program.
    echo "FAIL! i can't run until i have a task directory." >&2
    exit 1
  fi
fi

# set the task dir.
taskdir=${taskpwd}/outputs/${taskdate}
runconf=${taskpwd}/outputs/conf/${taskname}-${runtime}
jobconf=${taskpwd}/outputs/job/${taskname}-${runtime}

# operate differently, depending on the task name.
if [ "${taskname}" == "space" ]; then
  # get the dimset.
  dimset="${3}"
  if [ "${dimset}" == "" ]; then dimset="NULL"; fi

  # see if we have the output already.
  if [ -e ${taskdir}/uids.csv \
       -o -e ${taskdir}/dims.csv \
       -o -e ${taskdir}/space.csv \
       -o -e ${taskdir}/dist.csv ]; then
    echo -n "space outputs already exist. overwrite? [n] "
    read resp

    # check the response.
    if [ "${resp}" == "y" -o "${resp}" == "Y" -o "${resp}" == "yes" ]; then
      # make the directory.
      rm -f ${taskdir}/uids.csv ${taskdir}/dims.csv
      rm -f ${taskdir}/space.csv ${taskdir}/dist.csv
    else exit; fi
  fi

  # try to guess the time.
  hours="01"
  if [ ! "$(echo ${dimset} | grep -e NULL -e bindingdb)" == "" ]; then
    hours="08"
  fi

  # set up the config file.
  cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=${taskdir}/space.csv
dist=${taskdir}/dist.csv
dimset=${dimset}
EOF

  # set up the job file.
  cat > ${jobconf} << EOF
#!/bin/sh
#PBS -N ${taskname}-${runtime}
#PBS -l nodes=1:ppn=8:ram32gb
#PBS -l walltime=${hours}:00:00
#PBS -o ${jobconf}.out.txt
#PBS -e ${jobconf}.err.txt
export LD_LIBRARY_PATH=\${LD_LIBRARY_PATH}:${gsl}/usr/lib
${taskpwd}/gen-${taskname}.x ${runconf}
EOF

  # queue up the job.
  qsub ${jobconf}
elif [ "${taskname}" == "pca-svd" ]; then
  # get the dimensionality variable.
  L="${3}"
  if [ "${L}" == "" ]; then L="3"; fi

  # see if we have the output already.
  if [ -e ${taskdir}/pca-svd.csv -o -e ${taskdir}/pca-svd.csv ]; then
    echo -n "princomp outputs already exist. overwrite? [n] "
    read resp

    # check the response.
    if [ "${resp}" == "y" -o "${resp}" == "Y" -o "${resp}" == "yes" ]; then
      # make the directory.
      rm -f ${taskdir}/pca-svd.csv ${taskdir}/pca-svd.txt
    else exit; fi
  fi

  # set up the config file.
  cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=${taskdir}/space.csv
dist=NULL
princomp=${taskdir}/pca-svd.csv
L=${L}
EOF

  # set up the job file.
  cat > ${jobconf} << EOF
#!/bin/sh
#PBS -N ${taskname}-${runtime}
#PBS -l nodes=1:ppn=16:ram32gb
#PBS -l walltime=180:00:00
#PBS -o ${jobconf}.out.txt
#PBS -e ${jobconf}.err.txt
export LD_LIBRARY_PATH=\${LD_LIBRARY_PATH}:${gsl}/usr/lib
${taskpwd}/gen-${taskname}.x ${runconf}
sed -e 's/,/ /g' ${taskdir}/pca-svd.csv > ${taskdir}/pca-svd.txt
rm -f ${taskdir}/princomp.csv ${taskdir}/princomp.txt
ln -sf pca-svd.csv ${taskdir}/princomp.csv
ln -sf pca-svd.txt ${taskdir}/princomp.txt
EOF

  # queue up the job.
  qsub ${jobconf}
elif [ "${taskname}" == "pca-pls" ]; then
  # get the dimensionality variable.
  L="${3}"
  if [ "${L}" == "" ]; then L="3"; fi

  # get the scaling method.
  scaling="${4}"
  if [ "${scaling}" == "" ]; then scaling="none"; fi

  # see if we have the output already.
  if [ -e ${taskdir}/pca-pls.csv -o -e ${taskdir}/pca-pls.csv ]; then
    echo -n "princomp outputs already exist. overwrite? [n] "
    read resp

    # check the response.
    if [ "${resp}" == "y" -o "${resp}" == "Y" -o "${resp}" == "yes" ]; then
      # make the directory.
      rm -f ${taskdir}/pca-pls.csv ${taskdir}/pca-pls.txt
    else exit; fi
  fi

  # set up the config file.
  cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=${taskdir}/space.csv
dist=NULL
princomp=${taskdir}/pca-pls.csv
L=${L}
scale=${scaling}
EOF

  # set up the job file.
  cat > ${jobconf} << EOF
#!/bin/sh
#PBS -N ${taskname}-${runtime}
#PBS -l nodes=1:ppn=8:ram32gb
#PBS -l walltime=01:00:00
#PBS -o ${jobconf}.out.txt
#PBS -e ${jobconf}.err.txt
export LD_LIBRARY_PATH=\${LD_LIBRARY_PATH}:${gsl}/usr/lib
${taskpwd}/gen-${taskname}.x ${runconf}
sed -e 's/,/ /g' ${taskdir}/pca-pls.csv > ${taskdir}/pca-pls.txt
if [ ! -e ${taskdir}/princomp.csv -a ! -e ${taskdir}/princomp.txt ]; then
  ln -sf pca-pls.csv ${taskdir}/princomp.csv
  ln -sf pca-pls.txt ${taskdir}/princomp.txt
fi
EOF

  # queue up the job.
  qsub ${jobconf}
elif [ "${taskname}" == "cluster" ]; then
  # get the clustering variable.
  K="${3}"

  # see if we have the output already.
  if [ -e ${taskdir}/cluster-k${K}.csv ]; then
    echo -n "cluster outputs already exist. overwrite? [n] "
    read resp

    # check the response.
    if [ "${resp}" == "y" -o "${resp}" == "Y" -o "${resp}" == "yes" ]; then
      # make the directory.
      rm -f ${taskdir}/cluster-k${K}.csv
    else exit; fi
  fi

  # set up the config file.
  cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=NULL
dist=NULL
princomp=${taskdir}/princomp.csv
cluster=${taskdir}/cluster-k${K}.csv
parms=${taskdir}/parms-k${K}.csv
K=${K}
EOF

  # set up the job file.
  cat > ${jobconf} << EOF
#!/bin/sh
#PBS -N ${taskname}-${runtime}
#PBS -l select=1
#PBS -l walltime=00:05:00
#PBS -o ${jobconf}.out.txt
#PBS -e ${jobconf}.err.txt
export LD_LIBRARY_PATH=\${LD_LIBRARY_PATH}:${gsl}/usr/lib
${taskpwd}/gen-${taskname}.x ${runconf}
EOF

  # queue up the job.
  qsub ${jobconf}
elif [ "${taskname}" == "clusters-serial" ]; then
  # loop through the cluster counts.
  for k in $(seq -f '%02g' 1 ${myk}); do
    # make sure we haven't already calculated this k.
    if [ -e ${taskdir}/best-cluster-k${k}.csv ]; then
      continue
    fi

    # set up the conf and job filenames.
    runconf=${taskpwd}/outputs/conf/${taskname}-${runtime}-${k}
    jobconf=${taskpwd}/outputs/job/${taskname}-${runtime}-${k}

    # set up the conf file.
    cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=NULL
dist=NULL
princomp=${taskdir}/princomp.csv
cluster=${taskdir}/best-cluster-k${k}.csv
parms=${taskdir}/best-parms-k${k}.csv
dispersion=${taskdir}/disp-k${k}.txt
K=${k}
R=${myrun}
EOF

    # set up the job file.
    cat > ${jobconf} << EOF
#!/bin/sh
#PBS -N ${taskname}-${k}
#PBS -l select=1
#PBS -l walltime=120:00:00
#PBS -o ${jobconf}.out.txt
#PBS -e ${jobconf}.err.txt
export LD_LIBRARY_PATH=\${LD_LIBRARY_PATH}:${gsl}/usr/lib
${taskpwd}/gen-clusters.x ${runconf}
EOF

    # queue up the job.
    sync
    qsub ${jobconf}
  done
elif [ "${taskname}" == "clusters-pbs-noarray" ]; then
  # loop through the cluster counts.
  for k in $(seq -f '%02g' ${myk}); do
    # make sure we haven't already calculated this k.
    if [ -e ${taskdir}/best-cluster-k${k}.csv ]; then
      continue
    fi

    # loop through the run counts.
    for run in $(seq -f '%03g' ${myrun}); do
      # set up the conf and job filenames.
      jobconf=${taskpwd}/outputs/job/${taskname}-${runtime}-${k}-${run}
      runconf=${taskpwd}/outputs/conf/${taskname}-${runtime}-${k}-${run}

      # set up the conf file
      cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=NULL
dist=NULL
princomp=${taskdir}/princomp.csv
cluster=${taskdir}/cluster-k${k}-${run}.csv
parms=${taskdir}/parms-k${k}-${run}.csv
K=${k}
EOF

      # set up the job file.
      cat > ${jobconf} << EOF
#!/bin/sh
#PBS -N ${taskname}-${k}-${run}
#PBS -l select=1
#PBS -l walltime=00:05:00
#PBS -o ${jobconf}.out.txt
#PBS -e ${jobconf}.err.txt
export LD_LIBRARY_PATH=\${LD_LIBRARY_PATH}:${gsl}/usr/lib
${taskpwd}/gen-cluster.x ${runconf}
EOF

      # queue up the job.
      sync
      qsub ${jobconf}
    done
  done
elif [ "${taskname}" == "clusters-pbs-array" ]; then
  # calculate the job count.
  jobs_start=0
  jobs_end=$(expr ${myrun} - 1)

  # loop through the cluster counts.
  for k in $(seq -f '%02g' ${myk}); do
    # make sure we haven't already calculated this k.
    if [ -e ${taskdir}/best-cluster-k${k}.csv ]; then
      continue
    fi

    # set up the conf and job filenames.
    jobconf=${taskpwd}/outputs/job/${taskname}-${runtime}-${k}

    # set up the job file.
    cat > ${jobconf} << EOF
#!/bin/sh
#PBS -t ${jobs_start}-${jobs_end}
#PBS -N ${taskname}-${k}
#PBS -l select=1
#PBS -l walltime=01:00:00
#PBS -o ${jobconf}.out.txt
#PBS -e ${jobconf}.err.txt
export LD_LIBRARY_PATH=\${LD_LIBRARY_PATH}:${gsl}/usr/lib
run=\$(seq -f '%03g' \${PBS_ARRAYID} \${PBS_ARRAYID})
runconf=${taskpwd}/outputs/conf/${taskname}-${runtime}-${k}-\${run}
cat > \${runconf} << EEOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=NULL
dist=NULL
princomp=${taskdir}/princomp.csv
cluster=${taskdir}/cluster-k${k}-\${run}.csv
parms=${taskdir}/parms-k${k}-\${run}.csv
K=${k}
EEOF
${taskpwd}/gen-cluster.x \${runconf}
EOF

    # queue up the job.
    sync
    qsub ${jobconf}

    # wait for the job to complete.
    #job_wait ${jobid}
    s=$(qstat -u bworley)
    while [ ! "${s}" == "" ]; do
      echo "waiting for jobs to complete..."
      sleep 15s
      s=$(qstat -u bworley)
    done
  done
elif [ "${taskname}" == "clusters-pbs-array-single" ]; then
  # calculate the job count.
  jobs_start=0
  jobs_end=$(expr ${myrun} - 1)

  # dont loop through the cluster counts.
  if [ "${3}" == "" ]; then
    echo "FAIL! i expect a cluster count!" >&2
    exit 1
  fi
  k=$(seq -f '%02g' ${3} ${3})

  # set up the conf and job filenames.
  jobconf=${taskpwd}/outputs/job/${taskname}-${runtime}-${k}

  # set up the job file.
  cat > ${jobconf} << EOF
#!/bin/sh
#PBS -t ${jobs_start}-${jobs_end}
#PBS -N ${taskname}-${k}
#PBS -l select=1
#PBS -l walltime=12:00:00
#PBS -o ${jobconf}.out.txt
#PBS -e ${jobconf}.err.txt
export LD_LIBRARY_PATH=\${LD_LIBRARY_PATH}:${gsl}/usr/lib
run=\$(seq -f '%03g' \${PBS_ARRAYID} \${PBS_ARRAYID})
runconf=${taskpwd}/outputs/conf/${taskname}-${runtime}-${k}-\${run}
if [ -e ${taskdir}/cluster-k${k}-\${run}.csv ]; then exit; fi
wait=\$(expr \${RANDOM} % ${myrun})
sleep \${wait}s
cat > \${runconf} << EEOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=NULL
dist=NULL
princomp=${taskdir}/princomp.csv
cluster=${taskdir}/cluster-k${k}-\${run}.csv
parms=${taskdir}/parms-k${k}-\${run}.csv
K=${k}
EEOF
${taskpwd}/gen-cluster.x \${runconf}
EOF

  # queue up the job.
  sync
  qsub ${jobconf}
elif [ "${taskname}" == "clusters-condor" ]; then
  # set up the job filename.
  echo -n "Building job(s)"
  jobconf=${taskpwd}/outputs/job/${taskname}-${runtime}
  cat > ${jobconf} << EOF
Universe = vanilla
Environment = "LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:${gsl}/usr/lib"
Executable = ${taskpwd}/gen-cluster.x
EOF

  # loop through the cluster counts.
  for k in $(seq -f '%02g' ${myk}); do
    # loop through the run counts.
    for run in $(seq -f '%03g' ${myrun}); do
      # set up the conf filename.
      runconf=${taskpwd}/outputs/conf/${taskname}-${runtime}-${k}-${run}

      # set up the conf file.
      cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=NULL
dist=NULL
princomp=${taskdir}/princomp.csv
cluster=${taskdir}/cluster-k${k}-${run}.csv
parms=${taskdir}/parms-k${k}-${run}.csv
K=${k}
EOF

      # append the job to the job file.
      cat >> ${jobconf} << EOF
Arguments = ${runconf}
Output = ${jobconf}-${run}.out.txt
Error = ${jobconf}-${run}.err.txt
Log = ${jobconf}-${run}.log.txt
Queue
EOF

      # dot.
      echo -n "."
    done
  done

  # submit the job.
  echo
  condor_submit ${jobconf}
elif [ "${taskname}" == "best-cluster" ]; then
  # see if we have the output already.
  if [ -e ${taskdir}/dispersion.txt ]; then
    echo -n "best-cluster outputs already exist. overwrite? [n] "
    read resp

    # check the response.
    if [ "${resp}" == "y" -o "${resp}" == "Y" -o "${resp}" == "yes" ]; then
      # make the directory.
      rm -f ${taskdir}/dispersion.txt
    else exit; fi
  fi

  # loop through the clusters.
  for k in $(seq -w 1 ${myk}); do
    # set up the conf and job filenames.
    runconf=${taskpwd}/outputs/conf/${taskname}-${runtime}-${k}
    jobconf=${taskpwd}/outputs/job/${taskname}-${runtime}-${k}

    # set up the config file.
    cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=NULL
dist=NULL
princomp=${taskdir}/princomp.csv
clusters=${runconf}-list
EOF

    # set up the job file.
    cat > ${jobconf} << EOF
#!/bin/sh
#PBS -N ${taskname}-${k}
#PBS -l select=1
#PBS -l walltime=00:05:00
#PBS -o ${jobconf}.out.txt
#PBS -e ${jobconf}.err.txt
export LD_LIBRARY_PATH=\${LD_LIBRARY_PATH}:${gsl}/usr/lib
ls ${taskdir}/cluster-k${k}-*.csv > ${runconf}-list
bestout=\$(${taskpwd}/gen-${taskname}.x ${runconf})
bestfile=\$(echo \${bestout} | awk '{print \$1}')
bestdisp=\$(echo \${bestout} | awk '{print \$2,\$3,\$4}')
pushd ${taskdir} 2>&1 >/dev/null
  rm -f best-cluster-k${k}.csv best-parms-k${k}.csv cluster-k${k}.tar.gz
  cp -f \$(basename \${bestfile}) best-cluster-k${k}.csv
  cp -f \$(basename \$(echo \${bestfile} | sed -e 's/cluster/parms/g')) \
    best-parms-k${k}.csv
  tar cf cluster-k${k}.tar cluster-k${k}-*.csv parms-k${k}-*.csv
  gzip -9 cluster-k${k}.tar
  for oldfile in \$(cat ${runconf}-list); do
    rm -f \$(basename \${oldfile}) \
      \$(basename \$(echo \${oldfile} | sed -e 's/cluster/parms/g'))
  done
popd 2>&1 >/dev/null
echo "${k} \${bestdisp}" >> ${taskdir}/dispersion.txt
echo "\${bestdisp}" >> ${taskdir}/disp-k${k}.txt
EOF

    # queue up the job.
    sync
    qsub ${jobconf}
  done
elif [ "${taskname}" == "best-cluster-single" ]; then
  # dont loop through the cluster counts.
  k=$(seq -f '%02g' ${3} ${3})

  # see if we have the output already.
  if [ -e ${taskdir}/best-cluster-k${k}.csv ]; then
    echo -n "best-cluster-single outputs already exist. overwrite? [n] "
    read resp

    # check the response.
    if [ "${resp}" == "y" -o "${resp}" == "Y" -o "${resp}" == "yes" ]; then
      # make the directory.
      rm -f ${taskdir}/best-cluster-k${k}.csv
      rm -f ${taskdir}/best-parms-k${k}.csv
      rm -f ${taskdir}/disp-k${k}.txt
    else exit; fi
  fi

  # set up the config file.
  cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=NULL
dist=NULL
princomp=${taskdir}/princomp.csv
clusters=${runconf}-list
EOF

  # set up the job file.
  cat > ${jobconf} << EOF
#!/bin/sh
#PBS -N ${taskname}-${k}
#PBS -l select=1
#PBS -l walltime=00:05:00
#PBS -o ${jobconf}.out.txt
#PBS -e ${jobconf}.err.txt
export LD_LIBRARY_PATH=\${LD_LIBRARY_PATH}:${gsl}/usr/lib
ls ${taskdir}/cluster-k${k}-*.csv > ${runconf}-list
bestout=\$(${taskpwd}/gen-best-cluster.x ${runconf})
bestfile=\$(echo \${bestout} | awk '{print \$1}')
bestdisp=\$(echo \${bestout} | awk '{print \$2,\$3,\$4}')
pushd ${taskdir} 2>&1 >/dev/null
  rm -f best-cluster-k${k}.csv best-parms-k${k}.csv cluster-k${k}.tar.gz
  cp -f \$(basename \${bestfile}) best-cluster-k${k}.csv
  cp -f \$(basename \$(echo \${bestfile} | sed -e 's/cluster/parms/g')) \
    best-parms-k${k}.csv
  tar cf cluster-k${k}.tar cluster-k${k}-*.csv parms-k${k}-*.csv
  gzip -9 cluster-k${k}.tar
  for oldfile in \$(cat ${runconf}-list); do
    rm -f \$(basename \${oldfile}) \
      \$(basename \$(echo \${oldfile} | sed -e 's/cluster/parms/g'))
  done
popd 2>&1 >/dev/null
echo "\${bestdisp}" >> ${taskdir}/disp-k${k}.txt
EOF

  # queue up the job.
  sync
  qsub ${jobconf}
elif [ "${taskname}" == "status" ]; then
  t=$(echo "${myk}*${myrun}" | bc -l)
  for k in $(seq -f '%02g' 1 ${myk}); do
    echo -n " ${k}: ["
    for l in 03 05 10 15 20 25 30; do
      n=$(ls outputs/20110526-${l}/cluster-k${k}-*.csv 2>/dev/null | wc -l)
      n=$(seq -f '% 5g' ${n} ${n})
      echo -n "${n}"
    done
    echo " ]"
  done
  echo "-------------------------------------------"
  echo -n "tot: ["
  for l in 03 05 10 15 20 25 30; do
    n=$(ls outputs/20110526-${l}/cluster-k*-*.csv 2>/dev/null | wc -l)
    n=$(echo "scale=0;100.0*${n}/${t}" | bc -l)
    n=$(seq -f '% 4g' ${n} ${n})
    echo -n "${n}%"
  done
  echo " ]"
elif [ "${taskname}" == "killall" ]; then
  # get the jobs and remove them in one swift stroke.
  qstat -u bworley | sed -n '/^[0-9]/p' | \
    sed -e 's/^\([0-9].*\)\.[a-z].*/\1/g' | xargs qdel
  condor_q bworley | awk '$2=="bworley"{print $1}' | xargs condor_rm
else
  # output an error message.
  echo "FAIL! unknown task name." >&2
  exit 1
fi
