#!/bin/bash

# get the task name.
taskname="${1}"
taskdate="${2}"

# get the task pwd.
if [ "$(hostname)" == "prairiefire" ]; then
  echo "FAIL! you can't run tasks on the prairiefire login node!" >&2
  exit 1
elif [ "$(hostname)" == "bionmr-c1.unl.edu" \
     -o ! "$(hostname | grep node)" == "" ]; then
  taskpwd=/home/DATA/bworley/research/borg/classifiers/rev-b
else
  taskpwd=/DATA/bworley/research/borg/classifiers/rev-b
fi

# source the include file.
source ${taskpwd}/task-include.sh

# see if we ran without arguments.
if [ "${taskname}" == "" -a "${taskdate}" == "" ]; then
  task_info_list
fi

# make sure we're in the task pwd.
if [ ! "$(pwd)" == "${taskpwd}" ]; then cd ${taskpwd}; fi
taskapp=${taskpwd}/$(basename ${0})

# check the outputs directory.
if [ ! -d outputs ]; then mkdir outputs; fi

# check the task date.
if [ "${taskdate}" == "" ]; then
  # we need a task directory.
  echo "INFO: you must specify a task date as a second argument." >&2
  task_info ${taskname}
elif [ ! -d outputs/${taskdate} ]; then
  # ask if we wish to create a new task directory.
  echo -n "task directory outputs/${taskdate} does not exist. create? [n] "
  read resp

  # check the response.
  if [ "${resp}" == "y" -o "${resp}" == "Y" -o "${resp}" == "yes" ]; then
    # make the directory.
    mkdir outputs/${taskdate}
  else
    # leave the program.
    echo "FAIL! i can't run until i have a task directory." >&2
    exit 1
  fi
fi

# set the task dir.
taskdir=outputs/${taskdate}
runconf=${taskdir}/conf-${taskname}-${runtime}

# operate differently, depending on the task name.
if [ "${taskname}" == "space" ]; then
  # get the dimset.
  dimset="${3}"
  if [ "${dimset}" == "" ]; then dimset="NULL"; fi

  # see if we have the output already.
  if [ -e ${taskdir}/uids.csv \
       -o -e ${taskdir}/dims.csv \
       -o -e ${taskdir}/space.csv \
       -o -e ${taskdir}/dist.csv ]; then
    echo -n "space outputs already exist. overwrite? [n] "
    read resp

    # check the response.
    if [ "${resp}" == "y" -o "${resp}" == "Y" -o "${resp}" == "yes" ]; then
      # make the directory.
      rm -f ${taskdir}/uids.csv ${taskdir}/dims.csv
      rm -f ${taskdir}/space.csv ${taskdir}/dist.csv
    else exit; fi
  fi

  # set up the config file.
  cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=${taskdir}/space.csv
dist=${taskdir}/dist.csv
dimset=${dimset}
EOF

  # run the space program.
  ./gen-space.x ${runconf} 2>&1 | tee ${taskdir}/output-space.txt
elif [ "${taskname}" == "bispace" ]; then
  # get the dimset.
  dimset="${3}"
  if [ "${dimset}" == "" ]; then dimset="NULL"; fi

  # see if we have the output already.
  if [ -e ${taskdir}/bispace.csv ]; then
    echo -n "bispace outputs already exist. overwrite? [n] "
    read resp

    # check the response.
    if [ "${resp}" == "y" -o "${resp}" == "Y" -o "${resp}" == "yes" ]; then
      # make the directory.
      rm -f ${taskdir}/bispace.csv
    else exit; fi
  fi

  # set up the config file.
  cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=${taskdir}/space.csv
dist=${taskdir}/dist.csv
biuids=${taskdir}/biuids.csv
bidims=${taskdir}/bidims.csv
bispace=${taskdir}/bispace.csv
dimset=${dimset}
EOF

  # run the space program.
  ./gen-bispace.x ${runconf} 2>&1 | tee ${taskdir}/output-bispace.txt
elif [ "${taskname}" == "pruned" ]; then
  # see if we have the output already.
  if [ -e ${taskdir}/dims.csv.p \
       -o -e ${taskdir}/space.csv.p \
       -o -e ${taskdir}/dist.csv.p ]; then
    echo -n "pruned outputs already exist. overwrite? [n] "
    read resp

    # check the response.
    if [ "${resp}" == "y" -o "${resp}" == "Y" -o "${resp}" == "yes" ]; then
      # make the directory.
      rm -f ${taskdir}/dims.csv.p
      rm -f ${taskdir}/space.csv.p ${taskdir}/dist.csv.p
    else exit; fi
  fi

  # set up the config file.
  cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=${taskdir}/space.csv
dist=${taskdir}/dist.csv
EOF

  # run the pruned program.
  ./gen-pruned.x ${runconf} 2>&1 | tee ${taskdir}/output-pruned.txt
elif [ "${taskname}" == "pca-svd" ]; then
  # get the dimensionality variable.
  L="${3}"
  if [ "${L}" == "" ]; then L="3"; fi

  # see if we have the output already.
  if [ -e ${taskdir}/pca-svd.csv -o -e ${taskdir}/pca-svd.csv ]; then
    echo -n "princomp outputs already exist. overwrite? [n] "
    read resp

    # check the response.
    if [ "${resp}" == "y" -o "${resp}" == "Y" -o "${resp}" == "yes" ]; then
      # make the directory.
      rm -f ${taskdir}/pca-svd.csv ${taskdir}/pca-svd.txt
    else exit; fi
  fi

  # set up the config file.
  cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=${taskdir}/space.csv
dist=NULL
princomp=${taskdir}/pca-svd.csv
L=${L}
EOF

  # run the princomp program.
  ./gen-pca-svd.x ${runconf} 2>&1 | tee ${taskdir}/output-pca-svd.txt
  sed -e 's/,/ /g' ${taskdir}/pca-svd.csv > ${taskdir}/pca-svd.txt
  rm -f ${taskdir}/princomp.csv ${taskdir}/princomp.txt
  ln -sf pca-svd.csv ${taskdir}/princomp.csv
  ln -sf pca-svd.txt ${taskdir}/princomp.txt
elif [ "${taskname}" == "pca-pls" ]; then
  # get the dimensionality variable.
  L="${3}"
  if [ "${L}" == "" ]; then L="3"; fi

  # get the scaling method.
  scaling="${4}"
  if [ "${scaling}" == "" ]; then scaling="none"; fi

  # see if we have the output already.
  if [ -e ${taskdir}/pca-pls.csv -o -e ${taskdir}/pca-pls.csv ]; then
    echo -n "princomp outputs already exist. overwrite? [n] "
    read resp

    # check the response.
    if [ "${resp}" == "y" -o "${resp}" == "Y" -o "${resp}" == "yes" ]; then
      # make the directory.
      rm -f ${taskdir}/pca-pls.csv ${taskdir}/pca-pls.txt
    else exit; fi
  fi

  # set up the config file.
  cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=${taskdir}/space.csv
dist=NULL
princomp=${taskdir}/pca-pls.csv
conts=${taskdir}/conts.csv
L=${L}
scale=${scaling}
EOF

  # run the princomp program.
  ./gen-pca-pls.x ${runconf} 2>&1 | tee ${taskdir}/output-pca-pls.txt

  # make a non-csv version of the pca scores for plotting purposes.
  sed -e 's/,/ /g' ${taskdir}/pca-pls.csv > ${taskdir}/pca-pls.txt
  if [ ! -e ${taskdir}/princomp.csv -a ! -e ${taskdir}/princomp.txt ]; then
    ln -sf pca-pls.csv ${taskdir}/princomp.csv
    ln -sf pca-pls.txt ${taskdir}/princomp.txt
  fi

  # make a conts.txt file.
  sed -e 's/,/ /g' ${taskdir}/conts.csv > ${taskdir}/conts.txt
elif [ "${taskname}" == "dimconts" ]; then
  # see if we have outputs already.
  if [ -e ${taskdir}/dimconts.csv -o -e ${taskdir}/dimconts.txt ]; then
    echo -n "dimconts outputs already exist. overwrite? [n] "
    read resp

    # check the response.
    if [ "${resp}" == "y" -o "${resp}" == "Y" -o "${resp}" == "yes" ]; then
      # make the directory.
      rm -f ${taskdir}/dimconts.csv ${taskdir}/dimconts.txt
    else exit; fi
  fi

  # loop through the dimension types.
  echo " === CALCULATING TOTAL CONTRIBUTIONS ==="
  for dim in $(cut -f 2 -d , ${taskdir}/dims.csv | \
               cut -f 1 -d : | sort -u); do
    # loop through the dims again.
    echo -n "${dim}: "
    dimtotvar="0.0"
    for dimidx in $(sed -e 's@[:,]@ @g' ${taskdir}/dims.csv | \
                    awk "\$2 == \"${dim}\" { print \$1 }"); do
      dimvar=$(awk "\$1 == ${dimidx} { print \$2 }" ${taskdir}/conts.txt)
      dimtotvar=$(echo "${dimtotvar} + ${dimvar}" | bc -l)
    done
    echo ${dimtotvar}
    echo "${dim},${dimtotvar}" >> ${taskdir}/dimconts.csv
  done
elif [ "${taskname}" == "cluster" ]; then
  # get the clustering variable.
  K="${3}"

  # see if we have the output already.
  if [ -e ${taskdir}/cluster-k${K}.csv ]; then
    echo -n "cluster outputs already exist. overwrite? [n] "
    read resp

    # check the response.
    if [ "${resp}" == "y" -o "${resp}" == "Y" -o "${resp}" == "yes" ]; then
      # make the directory.
      rm -f ${taskdir}/cluster-k${K}.csv
    else exit; fi
  fi

  # set up the config file.
  cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=NULL
dist=NULL
princomp=${taskdir}/princomp.csv
cluster=${taskdir}/cluster-k${K}.csv
parms=${taskdir}/parms-k${K}.csv
K=${K}
EOF

  # run the cluster program.
  ./gen-cluster.x ${runconf} 2>&1 | tee ${taskdir}/output-cluster-k${K}.txt
elif [ "${taskname}" == "clusters" ]; then
  # loop through the cluster counts.
  for k in $(seq -f '%02g' ${myk}); do
    # set up the config file.
    cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=NULL
dist=NULL
princomp=${taskdir}/princomp.csv
cluster=${taskdir}/best-cluster-k${k}.csv
parms=${taskdir}/best-parms-k${k}.csv
dispersion=${taskdir}/disp-k${k}.txt
K=${k}
R=${myrun}
EOF

    # run the clusters program.
    ./gen-clusters.x ${runconf} 2>&1 | tee ${taskdir}/output-clusters-k${k}.txt
  done

  # output the dispersion file.
  cat ${taskdir}/disp-k*.txt | sort -n > ${taskdir}/dispersion.txt
elif [ "${taskname}" == "clusters-nodes" ]; then
  # check the hostname.
  if [ "$(hostname)" == "bionmr-c1.unl.edu" ]; then
    # cluster head node.
    for k in $(seq -w 1 ${myk}); do
      # remote shell the task command.
      node=$(echo "scale=0;((${k}-1)%16)+1" | bc -l)
      node=$(seq -f '%03g' ${node} ${node})
      rsh node${node} "${taskapp} $* ${k}" &
    done
  elif [ ! "$(hostname | grep node)" == "" ]; then
    # get the k-value.
    k=$(seq -f '%02g' ${3} ${3})

    # set up the config file.
    cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=NULL
dist=NULL
princomp=${taskdir}/princomp.csv
cluster=${taskdir}/best-cluster-k${k}.csv
parms=${taskdir}/best-parms-k${k}.csv
dispersion=${taskdir}/disp-k${k}.txt
K=${k}
R=${myrun}
EOF

    # run the clusters program.
    ${taskdir}/gen-clusters-$(hostname).x ${runconf} 1>/dev/null \
      2>${taskdir}/output-clusters-k${k}.txt
  else
    # other. probably a workstation.
    ssh bionmr-c1 /home${taskapp} $*
  fi
elif [ "${taskname}" == "best-cluster" ]; then
  # see if we have the output already.
  if [ -e ${taskdir}/dispersion.txt ]; then
    echo -n "best-cluster outputs already exist. overwrite? [n] "
    read resp

    # check the response.
    if [ "${resp}" == "y" -o "${resp}" == "Y" -o "${resp}" == "yes" ]; then
      # make the directory.
      rm -f ${taskdir}/dispersion.txt
    else exit; fi
  fi

  # loop through the clusters.
  echo "INFO: finding optimal clustering results..."
  for k in $(seq -w 1 ${myk}); do
    # print a nice message.
    echo -n " k=${k}: "

    # set up the config file.
    cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=NULL
dist=NULL
princomp=${taskdir}/princomp.csv
clusters=${runconf}-list
EOF

    # get the list of cluster files.
    ls ${taskdir}/cluster-k${k}-*-*.csv > ${runconf}-list

    # run the cluster program.
    bestout=$(./gen-best-cluster.x ${runconf} \
      2>${taskdir}/output-best-cluster-k${k}.txt)
    bestfile=$(echo ${bestout} | awk '{print $1}')
    bestdisp=$(echo ${bestout} | awk '{print $2,$3,$4}')

    # move the best file.
    pushd ${taskdir} 2>&1 >/dev/null
    rm -f best-cluster-k${k}.csv best-parms-k${k}.csv
    cp -f $(basename ${bestfile}) best-cluster-k${k}.csv
    cp -f $(basename $(echo ${bestfile} | sed -e 's/cluster/parms/g')) \
      best-parms-k${k}.csv
    popd 2>&1 >/dev/null

    # clean up the other files.
    rm -f ${runconf}-list

    # print the best clustering.
    echo ${bestfile}

    # add the dispersion statistics to the output file.
    echo "${k} ${bestdisp}" >> ${taskdir}/dispersion.txt
  done
elif [ "${taskname}" == "entropy" ]; then
  # see if we have the output already.
  if [ -e ${taskdir}/entropy.txt ]; then
    echo -n "entropy outputs already exist. overwrite? [n] "
    read resp

    # check the response.
    if [ "${resp}" == "y" -o "${resp}" == "Y" -o "${resp}" == "yes" ]; then
      # make the directory.
      rm -f ${taskdir}/entropy.txt
    else exit; fi
  fi

  # loop through the clusters.
  echo "INFO: calculating known protein clustering entropies..."
  for k in $(seq -w 1 ${myk}); do
    # print a nice message.
    echo -n " k=${k}: "

    # set up the config file.
    cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=NULL
dist=NULL
cluster=${taskdir}/best-cluster-k${k}.csv
EOF

    # run the entropy program.
    entropies=$(./gen-entropy.x ${runconf} \
      2>${taskdir}/output-entropy-k${k}.txt)

    # print the entropies.
    echo ${entropies}

    # add the dispersion statistics to the output file.
    echo "${k} ${entropies}" >> ${taskdir}/entropy.txt
  done
elif [ "${taskname}" == "network" ]; then
  # get the cluster count.
  K=$(seq -f '%02g' ${3} ${3})

  # loop through the clusters.
  echo "INFO: writing networks for clustering at ${K} clusters."
  for k in $(seq -f '%02g' 1 ${K}); do
    # set up the config file.
    cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=NULL
dist=${taskdir}/dist.csv
cluster=${taskdir}/best-cluster-k${K}.csv
network=${taskdir}/network-k${K}-c${k}.csv
K=${k}
EOF

    # run the network program.
    ./gen-network.x ${runconf} 2>${taskdir}/output-network-k${K}-c${k}.txt
  done
elif [ "${taskname}" == "distance" ]; then
  # get the cluster count.
  K=$(seq -f '%02g' ${3} ${3})

  # set up the config file.
  cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=${taskdir}/space.csv
dist=${taskdir}/dist.csv
cluster=${taskdir}/best-cluster-k${K}.csv
distance=${taskdir}/distance-k${K}.csv
EOF

  # run the distance program.
  ./gen-distance.x ${runconf} 2>&1 | tee ${taskdir}/output-distance-k${K}.txt
elif [ "${taskname}" == "consensus" ]; then
  # get the cluster count.
  K=$(seq -f '%02g' ${3} ${3})

  # loop through the clusters.
  for k in $(seq -f '%02g' 1 ${K}); do
    # set up the config file.
    cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=${taskdir}/space.csv
dist=NULL
cluster=${taskdir}/best-cluster-k${K}.csv
consensus=${taskdir}/consensus-k${K}-c${k}.html
K=${k}
EOF

    # run the distance program.
    ./gen-consensus.x ${runconf} 2>&1 | \
      tee ${taskdir}/output-consensus-k${K}.txt
  done
elif [ "${taskname}" == "weights" ]; then
  # get the cluster count.
  K=$(seq -f '%02g' ${3} ${3})

  # loop through the clusters.
  for k in $(seq -f '%02g' 1 ${K}); do
    # set up the config file.
    cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=${taskdir}/space.csv
dist=NULL
cluster=${taskdir}/best-cluster-k${K}.csv
weights=${taskdir}/weights-k${K}-c${k}.html
K=${k}
EOF

    # run the weights program.
    ./gen-weights.x ${runconf} 2>&1 | \
      tee ${taskdir}/output-weights-k${K}.txt
  done
elif [ "${taskname}" == "ellipse" ]; then
  # get the cluster count.
  K=$(seq -f '%02g' ${3} ${3})

  # set up the config file.
  cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=NULL
dist=NULL
parms=${taskdir}/best-parms-k${K}.csv.tmp
ellipse=${taskdir}/ellipse-k${K}.dat
EOF

  # fix up the parms.
  sed -e 's/^\([0-9.-]*\),\([0-9.-]*\),[^\[]*\[\([0-9.-]*\) \([0-9.-]*\)[^;]*; \([0-9.-]*\) \([0-9.-]*\).*\]$/\1 \2 \3 \4 \5 \6/g' \
    ${taskdir}/best-parms-k${K}.csv > ${taskdir}/best-parms-k${K}.csv.tmp

  # run the ellipse program.
  ./gen-ellipse.x ${runconf} 2>&1 | \
    tee ${taskdir}/output-ellipse-k${K}.txt

  # clean up.
  rm -f ${taskdir}/best-parms-k${K}.csv.tmp
elif [ "${taskname}" == "ellipses" ]; then
  # loop through the clusters.
  for k in $(seq -f '%02g' 1 ${myk}); do
    # set up the config file.
    cat > ${runconf} << EOF
uids=${taskdir}/uids.csv
dims=${taskdir}/dims.csv
space=NULL
dist=NULL
parms=${taskdir}/best-parms-k${k}.csv.tmp
ellipse=${taskdir}/ellipse-k${k}.dat
EOF

    # fix up the parms.
    sed -e 's/^\([0-9.-]*\),\([0-9.-]*\),[^\[]*\[\([0-9.-]*\) \([0-9.-]*\)[^;]*; \([0-9.-]*\) \([0-9.-]*\).*\]$/\1 \2 \3 \4 \5 \6/g' \
      ${taskdir}/best-parms-k${k}.csv > ${taskdir}/best-parms-k${k}.csv.tmp

    # run the ellipse program.
    ./gen-ellipse.x ${runconf} 2>&1 | \
      tee ${taskdir}/output-ellipse-k${k}.txt

    # clean up.
    rm -f ${taskdir}/best-parms-k${k}.csv.tmp
  done
elif [ "${taskname}" == "blast" ]; then
  # see if we have the output already.
  if [ -e ${taskdir}/blast.csv ]; then
    echo -n "blast outputs already exist. overwrite? [n] "
    read resp

    # check the response.
    if [ "${resp}" == "y" -o "${resp}" == "Y" -o "${resp}" == "yes" ]; then
      # make the directory.
      rm -f ${taskdir}/blast.csv
      rm -f ${taskdir}/database.fa ${taskdir}/database.fa.*
    else exit; fi
  fi

  # make a fasta file of the database proteins.
  q="use borg; select UID, sequence from uniprot where sequence is not null;"
  echo ${q} | mysql --password=mysqlpass | \
    sed -e '/UID/d' -e 's/^\(.*\)$/>gi|MYDB|\1/g' -e 's/\t/\n/g' \
      > ${taskdir}/database.fa
  makeblastdb -in ${taskdir}/database.fa -dbtype prot

  # loop through the proteins.
  N=$(wc -l ${taskdir}/uids.csv | cut -f 1 -d ' ')
  for i in $(seq 1 ${N}); do
    # get the uid.
    uid=$(awk -F , "\$1 == \"${i}\" {print \$2}" ${taskdir}/uids.csv)

    # get the sequence.
    q="use borg; select sequence from uniprot where UID = '${uid}';"
    seq=$(echo ${q} | mysql --password=mysqlpass | sed -e '/sequence/d')
    if [ "${seq}" == "NULL" ]; then continue; fi

    # make the fasta query file.
    echo "> ${uid}" > ${taskdir}/query.fa
    echo "${seq}" >> ${taskdir}/query.fa

    # run the blastp query.
    blastp -task blastp-short \
      -db ${taskdir}/database.fa \
      -query ${taskdir}/query.fa \
      -num_alignments 1 \
      -evalue 1000 \
      -outfmt 10 | sed -e "s/Query_1,gi|MYDB/${uid}/g" \
        >> ${taskdir}/blast.csv

    # remove the sequence.
    rm -f ${taskdir}/query.fa
  done

  # filter duplicates from the match list.
  awk -f task-blastfilt.awk ${taskdir}/blast.csv > ${taskdir}/blast.csv.tmp
  mv -f ${taskdir}/blast.csv.tmp ${taskdir}/blast.csv
elif [ "${taskname}" == "plot" ]; then
  # get the plot type.
  plotdir=${3}

  # see if this plot type exists.
  if [ ! -e figures/${plotdir} ]; then
    # fail.
    echo "FAIL! invalid plot directory." >&2
    exit 1
  fi

  # run the plot.
  echo "INFO: running figures/${plotdir}/gen.sh"
  pushd figures/${plotdir} 2>&1 >/dev/null
  ./gen.sh ${taskdir}
  popd 2>&1 >/dev/null
elif [ "${taskname}" == "rebuild" ]; then
  # check the hostname.
  if [ "$(hostname)" == "bionmr-c1.unl.edu" ]; then
    # cluster head node.
    for node in $(seq -w 1 ${mynode}); do
      # remote shell the task command.
      rm -f ${taskdir}/*.x
      rsh node0${node} "${taskapp} $*" &
    done
  elif [ ! "$(hostname | grep node)" == "" ]; then
    # rebuild code on the worker node.
    mkdir $(hostname); cd $(hostname)
    for cf in ../*.c; do ln -sf ${cf} $(basename ${cf}); done
    for hf in ../*.h; do ln -sf ${hf} $(basename ${hf}); done
    make -f ../Makefile again 2>&1 >/dev/null
    for xf in *.x; do
      mv -f ${xf} ../${taskdir}/$(basename ${xf} .x)-$(hostname).x
    done
    cd ..
    rm -rf $(hostname)
  else
    # other. probably a workstation.
    ssh bionmr-c1 /home${taskapp} $*
  fi
elif [ "${taskname}" == "killall" ]; then
  # check the hostname.
  if [ "$(hostname)" == "bionmr-c1.unl.edu" ]; then
    # cluster head node.
    for node in $(seq -w 1 ${mynode}); do
      # remote shell the task command.
      rsh node0${node} "${taskapp} $*"
    done
  elif [ ! "$(hostname | grep node)" == "" ]; then
    # kill the task and the running programs.
    killall -v gen-cluster-$(hostname).x
    killall -v gen-clusters-$(hostname).x
    killall -v task.sh
  else
    # other. probably a workstation.
    ssh bionmr-c1 /home${taskapp} $*
  fi
elif [ "${taskname}" == "prairiefire" ]; then
  scp *.[ch] *.sh *.awk *.awk.in *.txt README configure Makefile \
    prairiefire.unl.edu:/work/powers/bworley/classifiers/
elif [ "${taskname}" == "package" ]; then
  isodate=$(date +%Y%m%d)
  rm -f ../borg-classifier-${isodate}.tar.gz
  mkdir borg-classifier-${isodate}
  cp -r *.c *.h *.sh *.awk *.awk.in \
    AUTHORS COPYING INSTALL LICENSE README \
    configure Makefile figures \
    borg-classifier-${isodate}/
  tar cf ../borg-classifier-${isodate}.tar borg-classifier-${isodate}/
  gzip -9 ../borg-classifier-${isodate}.tar
  rm -rf borg-classifier-${isodate}
else
  # output an error message.
  echo "FAIL! unknown task name." >&2
  exit 1
fi

# delete the conf file.
rm -f ${runconf}
