
/*
 * borg-classifier/xlink.c: interaction network linkages source code
 * Copyright (C) 2011 Bradley Worley
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * 
 *     Free Software Foundation, Inc.
 *     59 Temple Place, Suite 330
 *     Boston, MA 02111-1307 USA
 */

/* core include. */
#include "core.h"
#if CONFIGURE_MYSQL == 1

/* the alias struct. */
unsigned long n_alias;
unsigned long *alias_i;
char **alias;

/* xlink_init: initializes structures used in crosslink formation. */
void xlink_init (void) {
  /* connect to the database. */
  db_connect (NULL);

  /* grab the aliases from the database. */
  fprintf (stderr, "INFO: querying database for alias information.\n");
  if (db_query ("select * from aliases;")) {
    /* fail out. */
    fprintf (stderr, "FAIL! unable to query aliases!\n");
    exit (1);
  }

  /* get the results. */
  struct db_results *results = db_results_get ();

  /* allocate some memory. */
  n_alias = results->rows;
  alias = (char **) malloc (sizeof (char *) * n_alias);
  alias_i = (unsigned long *) malloc (sizeof (unsigned long) * n_alias);

  /* save off the results. */
  unsigned long i;
  for (i = 0; i < n_alias; i++) {
    /* save the alias string. */
    alias[i] = str_dup (db_results_field (results, i, "alias"));

    /* get the index of the alias, if its in the database. */
    alias_i[i] = uid_index (db_results_field (results, i, "orig"));
  }

  /* free the results. */
  db_results_free (results);

  /* disconnect from the database. */
  db_disconnect ();
}

/* xlink_dealias: just like uid_index, but using the pre-loaded alias table. */
unsigned long xlink_dealias (const char *uid) {
  /* first, try looping through the list. */
  unsigned long idx = N + 16, i;
  for (i = 0; i < N; i++) {
    /* see if we have a match. */
    if (strcmp (uids[i], uid) == 0) {
      /* break out of the loop. */
      idx = i;
      break;
    }
  }

  /* see if we got it. */
  if (idx < N) return idx;

  /* didn't work. try the aliases array. */
  for (i = 0; i < n_alias; i++) {
    /* see if we have a match. */
    if (strcmp (alias[i], uid) == 0) {
      /* break out of the loop. */
      idx = alias_i[i];
      break;
    }
  }

  /* return the index value. */
  return idx;
}

/* xlink_get_string: gets string interaction partners for a given uid index. */
unsigned long *xlink_get (const char *t, unsigned long i, unsigned long *n) {
  /* declare required variables. */
  unsigned long j, k, ui, len = 0;
  unsigned long *link = NULL;
  char *uidA, *uidB;

  /* query the database using the straight uid. */
  db_connect (NULL);
  db_query ("select * from %s where actorA='%s' or actorB='%s';",
            t, uids[i], uids[i]);

  /* get the results. */
  struct db_results *results = db_results_get ();

  /* loop through the results. */
  for (k = 0; k < results->rows; k++) {
    /* get the uid strings. */
    uidA = db_results_field (results, k, "actorA");
    uidB = db_results_field (results, k, "actorB");

    /* see if we can find a link back into our dataset. */
    if (strcmp (uidA, uids[i]) == 0)
      ui = xlink_dealias (uidB);
    else
      ui = xlink_dealias (uidA);

    /* did we get another link? */
    if (ui < N) {
      /* see what to do with our link array. */
      len++;
      if (!link)
        link = (unsigned long *) malloc (sizeof (unsigned long) * len);
      else
        link = (unsigned long *) realloc (link, sizeof (unsigned long) * len);

      /* store the new index. */
      link[len - 1] = ui;
    }
  }

  /* free the results. */
  db_results_free (results);

  /* loop through the aliases. */
  for (j = 0; j < n_alias; j++) {
    /* see if we have a match to the uid index. */
    if (alias_i[j] == i) {
      /* query the database using the alias. */
      db_query ("select * from %s where actorA='%s' or actorB='%s';",
                t, alias[j], alias[j]);

      /* get the results. */
      results = db_results_get ();

      /* loop through the results. */
      for (k = 0; k < results->rows; k++) {
        /* get the uid strings. */
        uidA = db_results_field (results, k, "actorA");
        uidB = db_results_field (results, k, "actorB");

        /* see if we can find a link back into our dataset. */
        if (strcmp (uidA, alias[j]) == 0)
          ui = xlink_dealias (uidB);
        else
          ui = xlink_dealias (uidA);

        /* did we get another link? */
        if (ui < N) {
          /* see what to do with our link array. */
          len++;
          if (!link)
            link = (unsigned long *) malloc (sizeof (unsigned long) * len);
          else
            link = (unsigned long *)
              realloc (link, sizeof (unsigned long) * len);

          /* store the new index. */
          link[len - 1] = ui;
        }
      }

      /* free the results. */
      db_results_free (results);
    }
  }

  /* return the result. */
  db_disconnect ();
  *n = len;
  return link;
}

/* xlink_get_string: gets string interaction partners for a given uid index. */
unsigned long *xlink_get_string (unsigned long i, unsigned long *n) {
  /* use the metafunction. */
  return xlink_get ("string", i, n);
}

/* xlink_get_dip: gets dip interaction partners for a given uid index. */
unsigned long *xlink_get_dip (unsigned long i, unsigned long *n) {
  /* use the metafunction. */
  return xlink_get ("dip", i, n);
}

#endif
